import 'dart:html';

import 'package:flutter/material.dart';
import 'package:remove_emoji/remove_emoji.dart';
import 'package:flutter_markdown/flutter_markdown.dart';
import 'package:markdown/markdown.dart' as md;
import 'package:url_launcher/url_launcher.dart';
import 'package:google_fonts/google_fonts.dart';
import 'dart:convert';
import 'dart:ui' as ui;

class DialogBlock extends StatelessWidget {
  final String block;

  DialogBlock(this.block);

  List<Widget> renderBlock() {
    var json = jsonDecode(this.block);
    List<Widget> widgets = [];

    for (var block in json["blocks"]) {
      var blockType = block["type"];
      Widget widget;
      switch (blockType) {
        case "section":
          {
            widget = renderSectionBlock(block);
          }
          break;

        case "actions":
          {
            widget = renderActionsBlock(block);
          }
          break;

        case "divider":
          {
            widget = renderDividerBlock(block);
          }
          break;

        case "image":
          {
            widget = renderImageBlock(block);
          }
          break;

        case "context":
          {
            widget = renderContextBlock(block);
          }
          break;

        case "header":
          {
            widget = renderHeaderBlock(block);
          }
          break;
      }

      if (widget != null) {
        widgets.add(widget);
      }
    }

    return widgets;
  }

  Widget renderSectionBlock(block) {
    var tb = block["text"];
    var text = tb["text"];

    if (tb != null) {
      var hasEmoji = tb["emoji"];
      if (hasEmoji != null && hasEmoji == false) {
        text = RemoveEmoji().removemoji(text);
      }
      if (tb["type"] == "plain_text") {
        return Text(
          text,
          textAlign: TextAlign.left,
        );
      } else if (tb["type"] == "mrkdwn") {
        return Container(
          child: MarkdownBody(
            data: text,
            extensionSet: md.ExtensionSet(
              md.ExtensionSet.gitHubFlavored.blockSyntaxes,
              [md.EmojiSyntax(), ...md.ExtensionSet.gitHubFlavored.inlineSyntaxes],
            ),
            onTapLink: (text, href, title) {
              launch(href);
            },
          ),
        );
      }
    }
    return Container();
  }

  Widget renderActionsBlock(block) {
    List elements = block["elements"];
    List<Widget> actionWidgets = [];

    for (Map element in elements) {
      var text = element["text"];
      var hasEmoji = text["emoji"];
      var innerType = text["type"];
      if (hasEmoji != null && hasEmoji == false) {
        text["text"] = RemoveEmoji().removemoji(text["text"]);
      }

      switch (element["type"]) {
        case "button":
          Widget inner = Container();
          if (innerType == "plain_text") {
            inner = Container(
              child: Text(text["text"],
                  style:
                      GoogleFonts.getFont('Open Sans', fontSize: 14, fontWeight: FontWeight.w600, color: Colors.black)),
              padding: EdgeInsets.fromLTRB(8, 6, 8, 6),
              decoration: BoxDecoration(
                  border: Border.all(width: 0.5, color: Colors.grey.shade600),
                  borderRadius: BorderRadius.all(Radius.circular(4))),
            );
          } else if (innerType == "mrkdwn") {
            inner = MarkdownBody(
              data: text["text"],
              extensionSet: md.ExtensionSet(
                md.ExtensionSet.gitHubFlavored.blockSyntaxes,
                [md.EmojiSyntax(), ...md.ExtensionSet.gitHubFlavored.inlineSyntaxes],
              ),
              onTapLink: (text, href, title) {
                launch(href);
              },
            );
          }

          actionWidgets.add(Container(
              margin: EdgeInsets.fromLTRB(0, 8, 8, 8),
              child: TextButton(
                onPressed: () {},
                child: inner,
                style: ButtonStyle(overlayColor: MaterialStateProperty.all(Colors.grey.shade100)),
              )));
          break;

        default:
          break;
      }
    }

    return Row(
      children: actionWidgets,
    );
  }

  Widget renderDividerBlock(block) {
    return Divider(
      color: Colors.grey.shade500,
      height: 12,
      thickness: 0.5,
    );
  }

  Widget renderImageBlock(block) {
    var title = block["title"];
    var url = block["image_url"];
    var innerType = title != null ? title["type"] : "";

    var _image = ImageElement(src: url);
    // ignore: undefined_prefixed_name
    ui.platformViewRegistry.registerViewFactory(
      "_image",
      (int viewId) => _image,
    );

    Widget inner = Container();
    if (innerType == "plain_text") {
      var header = title["text"] != null ? title["text"] : "";

      inner = Container(
          alignment: Alignment.centerLeft,
          padding: EdgeInsets.fromLTRB(0, 8, 0, 4),
          child: Text(header,
              style: GoogleFonts.getFont('Open Sans',
                  fontSize: 14, fontWeight: FontWeight.w400, color: Colors.grey.shade600)));
    } else if (innerType == "mrkdwn") {
      inner = MarkdownBody(
        data: title["text"],
        extensionSet: md.ExtensionSet(
          md.ExtensionSet.gitHubFlavored.blockSyntaxes,
          [md.EmojiSyntax(), ...md.ExtensionSet.gitHubFlavored.inlineSyntaxes],
        ),
        onTapLink: (text, href, title) {
          launch(href);
        },
      );
    }

    return Container(
        child: Column(
      children: [
        inner,
        Container(
            alignment: Alignment.centerLeft,
            child: SizedBox(
                width: _image.width.toDouble(),
                height: _image.height.toDouble(),
                child: HtmlElementView(viewType: "_image"))),
      ],
    ));
  }

  Widget renderContextBlock(block) {
    return Container();
  }

  Widget renderHeaderBlock(block) {
    var text = block["text"];
    var innerType = text["type"];
    var innerText = text["text"] != null ? text["text"] : "";
    var hasEmoji = text["emoji"];
    if (hasEmoji != null && hasEmoji == false) {
      innerText = RemoveEmoji().removemoji(innerText);
    }

    Widget inner = Container();
    if (innerType == "plain_text") {
      inner = Container(
          alignment: Alignment.centerLeft,
          padding: EdgeInsets.fromLTRB(0, 8, 0, 4),
          child: Text(innerText,
              style:
                  GoogleFonts.getFont('Open Sans', fontSize: 18, fontWeight: FontWeight.w700, color: Colors.black87)));
    } else if (innerType == "mrkdwn") {
      inner = MarkdownBody(
        data: innerText,
        extensionSet: md.ExtensionSet(
          md.ExtensionSet.gitHubFlavored.blockSyntaxes,
          [md.EmojiSyntax(), ...md.ExtensionSet.gitHubFlavored.inlineSyntaxes],
        ),
        onTapLink: (text, href, title) {
          launch(href);
        },
      );
    }

    return inner;
  }

  @override
  Widget build(BuildContext context) {
    return Container(
      padding: EdgeInsets.all(16),
      child: Column(
        mainAxisAlignment: MainAxisAlignment.center,
        crossAxisAlignment: CrossAxisAlignment.start,
        children: renderBlock(),
      ),
    );
  }
}
